
import 'dart:convert';
import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:indigitall_flutter_plugin/core/api/requests/CoreRequest.dart';
import 'package:indigitall_flutter_plugin/core/api/responses/CoreResponse.dart';
import 'package:indigitall_flutter_plugin/core/utils/Log.dart';
import 'package:indigitall_flutter_plugin/core/utils/PrintUtils.dart';

class CoreClient {
  static final String TAG = "[IND]api.CoreClient";

  static const String _METHOD_GET = "GET";
  static const String _METHOD_POST = "POST";
  static const String _METHOD_PUT = "PUT";
  static const String _METHOD_DELETE = "DELETE";

  static Future<void> call(final String method, final String endpoint, final CoreRequest request, final CoreResponse? response) async {
    await callHttp(method, endpoint, request, response);

  }

  static Future<void> callHttp (final String method, final String endpoint, final CoreRequest request, final CoreResponse? response) async {
    Log log = new Log.withContext(null, TAG);
    HttpClient client = new HttpClient();
    final url = (Uri.parse(request.getPath(endpoint) + request.getParams()));
    HttpClientRequest? clientRequest;
    switch(method) {
      case _METHOD_PUT: {
        clientRequest = await client.putUrl(url);
      }
      break;

      case _METHOD_POST: {
        clientRequest = await client.postUrl(url);
      }
      break;

      case _METHOD_GET: {
        clientRequest = await client.getUrl(url);
      }
      break;
      case _METHOD_DELETE: {
        clientRequest = await client.deleteUrl(url);
      }
      break;
    }
    if (clientRequest != null){

      clientRequest.headers.set(HttpHeaders.contentTypeHeader, "application/json");
      if (Platform.isAndroid) clientRequest.headers.set("x-app-android", request.getPackageName());
      if (Platform.isIOS) clientRequest.headers.set("x-app-ios",request.getPackageName());
      if (method != _METHOD_GET){
        clientRequest.add(utf8.encode(json.encode(request.body)));
      }
      HttpClientResponse clientResponse = await clientRequest.close();
      int statusCode = clientResponse.statusCode;
      String responseMessage = await _readResponse(clientResponse).then((value) => value);

      if (statusCode >= 200 && statusCode < 300) {

        String responseBody = responseMessage;

        log.i("Method: " + method + "\n" +
            "URL: " + url.toString() + "\n")
            .d(_METHOD_GET != method ? "Request Body: " + PrintUtils.prettyPrintJson(request.getBody()) + "\n" : "")
            .i("Response Code: " + statusCode.toString() + "\n" +
            "Response Message: " + PrintUtils.prettyPrintJson(responseMessage) + "\n")
            .d("Response Body: " + PrintUtils.prettyPrintJson(responseBody))
            .writeLog();
      } else {
        log.i("Method: " + method + "\n" +
            "URL: " + url.toString() + "\n")
            .d(_METHOD_GET != method ? "Request Body: " + PrintUtils.prettyPrintJson(request.getBody()) + "\n" : "")
            .i("Response Code: " + statusCode.toString() + "\n" +
            "Error: " + PrintUtils.prettyPrintJson(responseMessage) + "\n")
            .writeLog();
      }

      if (response != null) {
        response.processData(statusCode, responseMessage, responseMessage);
      }
    }else{
      log.w("Is not a HttpsConnection: " + url.toString()).writeLog();
    }
    client.close();
  }

  static Future<String> _readResponse(HttpClientResponse response) {
    final completer = Completer<String>();
    final contents = StringBuffer();
    response.transform(utf8.decoder).listen((data) {
      contents.write(data);
    }, onDone: () => completer.complete(contents.toString()));
    return completer.future;
  }

  static Future<void> get(final String endpoint, final CoreRequest request, final CoreResponse? response) async {
    await call(_METHOD_GET, endpoint, request, response);
  }

  static Future<void> post(final String endpoint, final CoreRequest request, final CoreResponse? response) async {
    await call(_METHOD_POST, endpoint, request, response);
  }

  static Future<void> put(final String endpoint, final CoreRequest request, final CoreResponse? response) async {
    await call(_METHOD_PUT, endpoint, request, response);
  }

  static Future<void> delete(final String endpoint, final CoreRequest request, final CoreResponse? response) async {
    await call(_METHOD_DELETE, endpoint, request, response);
  }
}
